@@ -1,6 +1,6 @@
 /* whoopsie
  * 
- * Copyright © 2011-2012 Canonical Ltd.
+ * Copyright © 2011-2013 Canonical Ltd.
  * Author: Evan Dandrea <evan.dandrea@canonical.com>
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -46,6 +46,8 @@
 #include "connectivity.h"
 #include "monitor.h"
 #include "identifier.h"
+#include "logging.h"
+#include "globals.h"
 
 /* The length of time to wait before processing outstanding crashes, in seconds
  */
@@ -58,6 +60,9 @@ static gboolean online_state = TRUE;
 /* The URL of the crash database. */
 static char* crash_db_url = NULL;
 
+/* Username we will run under */
+static const char* username = "whoopsie";
+
 /*  The database identifier. Either:
  *  - The system UUID, taken from the DMI tables and SHA-512 hashed
  *  - The MAC address of the first non-loopback device, SHA-512 hashed */
@@ -66,9 +71,6 @@ static char* whoopsie_identifier = NULL;
 /* The URL for sending the initial crash report */
 static char* crash_db_submit_url = NULL;
 
-/* Username we will run under */
-static const char username[] = "whoopsie";
-
 /* The file path and descriptor for our instance lock */
 static const char* lock_path = "/var/lock/whoopsie/lock";
 static int lock_fd = 0;
@@ -77,13 +79,13 @@ static int lock_fd = 0;
 static const char* report_dir = "/var/crash";
 
 /* Options */
+int foreground = 0;
+static int assume_online = 0;
 
 #ifndef TEST
-static gboolean foreground = FALSE;
-
 static GOptionEntry option_entries[] = {
-    { "foreground", 'f', 0, G_OPTION_ARG_NONE, &foreground,
-      "Run in the foreground", NULL },
+    { "foreground", 'f', 0, G_OPTION_ARG_NONE, &foreground, "Run in the foreground", NULL },
+    { "assume-online", 'a', 0, G_OPTION_ARG_NONE, &assume_online, "Always assume there is a route to $CRASH_DB_URL.", NULL },
     { NULL }
 };
 #endif
@@ -264,12 +266,12 @@ upload_report (const char* message_data, int message_len, struct response_string
     /* TODO use curl_share for DNS caching. */
     /* Repeated calls to curl_global_init will have no effect. */
     if (curl_global_init (CURL_GLOBAL_SSL)) {
-        g_print ("Unable to initialize curl.\n\n");
+        log_msg ("Unable to initialize curl.\n\n");
         exit (EXIT_FAILURE);
     }
 
     if ((curl = curl_easy_init ()) == NULL) {
-        printf ("Couldn't init curl.\n");
+        log_msg ("Couldn't init curl.\n");
         return FALSE;
     }
     curl_easy_setopt (curl, CURLOPT_POST, 1);
@@ -288,9 +290,9 @@ upload_report (const char* message_data, int message_len, struct response_string
     curl_slist_free_all(list);
     curl_easy_getinfo (curl, CURLINFO_RESPONSE_CODE, &response_code);
 
-    printf ("Sent; server replied with: %s\n",
+    log_msg ("Sent; server replied with: %s\n",
         curl_easy_strerror (result_code));
-    printf ("Response code: %ld\n", response_code);
+    log_msg ("Response code: %ld\n", response_code);
     curl_easy_cleanup (curl);
 
     if (result_code != CURLE_OK)
@@ -515,7 +517,7 @@ upload_core (const char* uuid, const char* arch, const char* core_data) {
     /* TODO use CURLOPT_READFUNCTION to transparently compress data with
      * Snappy. */
     if ((curl = curl_easy_init ()) == NULL) {
-        printf ("Couldn't init curl.\n");
+        log_msg ("Couldn't init curl.\n");
         g_free (crash_db_core_url);
         return FALSE;
     }
@@ -536,9 +538,9 @@ upload_core (const char* uuid, const char* arch, const char* core_data) {
 
     curl_easy_getinfo (curl, CURLINFO_RESPONSE_CODE, &response_code);
 
-    printf ("Sent; server replied with: %s\n",
+    log_msg ("Sent; server replied with: %s\n",
         curl_easy_strerror (result_code));
-    printf ("Response code: %ld\n", response_code);
+    log_msg ("Response code: %ld\n", response_code);
     curl_easy_cleanup (curl);
     g_free (crash_db_core_url);
     destroy_response_string (&s);
@@ -568,11 +570,11 @@ handle_response (GHashTable* report, char* response_data)
                     /* We do not retry the upload. Once is a big enough hit to
                      * their Internet connection, and we can always count on
                      * the next person in line to send it. */
-                    printf ("Upload of the core dump failed.\n");
+                    log_msg ("Upload of the core dump failed.\n");
             } else
-                printf ("Asked for a core dump that we don't have.\n");
+                log_msg ("Asked for a core dump that we don't have.\n");
         } else
-            printf ("Got command: %s\n", command);
+            log_msg ("Got command: %s\n", command);
     }
 }
 
@@ -588,28 +590,28 @@ parse_and_upload_report (const char* crash_file)
     bson b[1];
     int response = 0;
 
-    g_print ("Parsing %s.\n", crash_file);
+    log_msg ("Parsing %s.\n", crash_file);
     report = parse_report (crash_file, FALSE, &error);
     if (!report) {
         if (error) {
-            g_print ("Unable to parse report (%s): %s\n", crash_file,
+            log_msg ("Unable to parse report (%s): %s\n", crash_file,
                        error->message);
             g_error_free (error);
         } else {
-            g_print ("Unable to parse report (%s)\n", crash_file);
+            log_msg ("Unable to parse report (%s)\n", crash_file);
         }
         /* Do not keep trying to parse and upload this */
         return TRUE;
     }
 
     if (!bsonify (report, b, &message_data, &message_len)) {
-        g_print ("Unable to bsonify report (%s)\n", crash_file);
+        log_msg ("Unable to bsonify report (%s)\n", crash_file);
         if (bson_data (b))
             bson_destroy (b);
         /* Do not keep trying to parse and upload this */
         success = TRUE;
     } else {
-        g_print ("Uploading %s.\n", crash_file);
+        log_msg ("Uploading %s.\n", crash_file);
         init_response_string (&s);
         response = upload_report (message_data, message_len, &s);
         if (bson_data (b))
@@ -624,8 +626,10 @@ parse_and_upload_report (const char* crash_file)
         else
             success = FALSE;
 
-        if (response > 200)
-            g_print ("Server replied with:\n%s\n", s.p);
+        if (response > 200) {
+            log_msg ("Server replied with:\n");
+            log_msg ("%s\n", s.p);
+        }
 
         if (response == 200 && s.length > 0)
             handle_response (report, s.p);
@@ -672,7 +676,7 @@ process_existing_files (const char* report_dir)
             continue;
         } else if (online_state && parse_and_upload_report (crash_file)) {
             if (!mark_handled (crash_file))
-                g_print ("Unable to mark report as seen (%s)\n", crash_file);
+                log_msg ("Unable to mark report as seen (%s)\n", crash_file);
 
         } 
 
@@ -691,7 +695,7 @@ void daemonize (void)
     struct rlimit rl = {0};
 
     if (getrlimit (RLIMIT_NOFILE, &rl) < 0) {
-        g_print ("Could not get resource limits.\n");
+        log_msg ("Could not get resource limits.\n");
         exit (EXIT_FAILURE);
     }
 
@@ -713,7 +717,7 @@ void daemonize (void)
     if ((open ("/dev/null", O_RDWR) != 0) ||
         (dup (0) != 1) ||
         (dup (0) != 2)) {
-        g_print ("Could not redirect file descriptors to /dev/null.\n");
+        log_msg ("Could not redirect file descriptors to /dev/null.\n");
         exit (EXIT_FAILURE);
     }
 }
@@ -730,21 +734,21 @@ exit_if_already_running (void)
         /* use system directory */
         if (mkdir ("/var/lock/whoopsie", 0755) < 0) {
             if (errno != EEXIST) {
-                g_print ("Could not create lock directory.\n");
+                log_msg ("Could not create lock directory.\n");
             }
         }
     }
 
-    g_print("Using lock path: %s\n", lock_path);
+    log_msg ("Using lock path: %s\n", lock_path);
 
     lock_fd = open (lock_path, O_CREAT | O_RDWR, 0600);
     rc = flock (lock_fd, LOCK_EX | LOCK_NB);
     if (rc) {
         if (EWOULDBLOCK == errno) {
-            g_print ("Another instance is already running.\n");
+            log_msg ("Another instance is already running.\n");
             exit (1);
         } else {
-            g_print ("Could not create lock file: %s\n", strerror (errno));
+            log_msg ("Could not create lock file: %s\n", strerror (errno));
         }
     }
 }
@@ -806,7 +810,7 @@ drop_privileges (GError** error)
 void
 network_changed (gboolean available)
 {
-    g_print (available ? "online\n" : "offline\n");
+    log_msg (available ? "online\n" : "offline\n");
     if (!available) {
         online_state = FALSE;
         return;
@@ -825,12 +829,12 @@ gboolean
 check_online_then_upload (const char* crash_file) {
 
     if (!online_state) {
-        g_print ("Not online; processing later (%s).\n", crash_file);
+        log_msg ("Not online; processing later (%s).\n", crash_file);
         return FALSE;
     }
 
     if (!parse_and_upload_report (crash_file)) {
-        g_print ("Could not upload; processing later (%s).\n", crash_file);
+        log_msg ("Could not upload; processing later (%s).\n", crash_file);
         return FALSE;
     }
 
@@ -838,31 +842,32 @@ check_online_then_upload (const char* crash_file) {
 }
 
 void
-create_directory (void)
+create_crash_directory (void)
 {
     struct passwd *pw = NULL;
 
     if (mkdir (report_dir, 0755) < 0) {
         if (errno != EEXIST) {
-            g_print ("Could not create non-existent directory to monitor (%d): %s.\n", errno, report_dir);
+            log_msg ("Could not create non-existent report_directory to monitor (%d): %s.\n", errno, report_dir);
             exit (EXIT_FAILURE);
         }
     } else {
         /* Only change the permissions if we've just created it */
         if (!(pw = getpwnam (username))) {
-            g_print ("Could not find user, %s.\n", username);
+            log_msg ("Could not find user, %s.\n", username);
             exit (EXIT_FAILURE);
         }
         if (chown (report_dir, -1, pw->pw_gid) < 0) {
-            g_print ("Could not change ownership of %s.\n", report_dir);
+            log_msg ("Could not change ownership of %s.\n", report_dir);
             exit (EXIT_FAILURE);
         }
         if (chmod (report_dir, 03777) < 0) {
-            g_print ("Could not change permissions on %s.\n", report_dir);
+            log_msg ("Could not change permissions on %s.\n", report_dir);
             exit (EXIT_FAILURE);
         }
     }
 }
+
 #ifndef TEST
 static GMainLoop* loop = NULL;
 
@@ -875,7 +880,7 @@ parse_arguments (int* argc, char** argv[])
     context = g_option_context_new (NULL);
     g_option_context_add_main_entries (context, option_entries, NULL);
     if (!g_option_context_parse (context, argc, argv, &err)) {
-        g_print ("whoopsie: %s\n", err->message);
+        log_msg ("whoopsie: %s\n", err->message);
         g_error_free (err);
         exit (EXIT_FAILURE);
     }
@@ -916,13 +921,16 @@ main (int argc, char** argv)
     parse_arguments (&argc, &argv);
 
     if ((crash_db_url = get_crash_db_url ()) == NULL) {
-        g_print ("Could not get crash database location.\n");
+        log_msg ("Could not get crash database location.\n");
         exit (EXIT_FAILURE);
     }
 
+    if (!foreground)
+        open_log ();
+
     /* environment might change report directory and identifier */
     env = g_getenv ("APPORT_REPORT_DIR");
-    if (env != NULL)
+    if (env != NULL && *env != '\0')
         report_dir = env;
 
     env = g_getenv ("CRASH_DB_IDENTIFIER");
@@ -932,7 +940,7 @@ main (int argc, char** argv)
         whoopsie_identifier_generate (&whoopsie_identifier, &err);
 
     if (err) {
-        g_print ("%s\n", err->message);
+        log_msg ("%s\n", err->message);
         g_error_free (err);
         err = NULL;
         crash_db_submit_url = strdup (crash_db_url);
@@ -942,11 +950,11 @@ main (int argc, char** argv)
                                                whoopsie_identifier);
     }
 
-    create_directory ();
+    create_crash_directory ();
 
     drop_privileges (&err);
     if (err) {
-        g_print ("%s\n", err->message);
+        log_msg ("%s\n", err->message);
         g_error_free (err);
         exit (EXIT_FAILURE);
     }
@@ -964,7 +972,9 @@ main (int argc, char** argv)
     if (!monitor)
         exit (EXIT_FAILURE);
 
-    monitor_connectivity (crash_db_url, network_changed);
+    if (!assume_online)
+        monitor_connectivity (crash_db_url, network_changed);
+
     process_existing_files (report_dir);
     g_timeout_add_seconds (PROCESS_OUTSTANDING_TIMEOUT,
                            (GSourceFunc) process_existing_files, (gpointer) report_dir);
@@ -973,7 +983,10 @@ main (int argc, char** argv)
     g_main_loop_run (loop);
 
     unmonitor_directory (monitor, check_online_then_upload);
-    unmonitor_connectivity ();
+    if (!assume_online)
+        unmonitor_connectivity ();
+
+    close_log ();
 
     g_unlink (lock_path);
     close (lock_fd);
